﻿using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows.Forms;

namespace WinFormsDataBindingExampleCS
{
    public partial class Form1 : Form, INotifyPropertyChanged
    {
        private string _textBoxText;

        public string TextBoxText
        {
            get
            {
                return _textBoxText;
            }
            set
            {
                if (_textBoxText == value)
                    return;
                _textBoxText = value;
                NotifyOfPropertyChange();
            }
        }

        private bool _checkBoxBool;

        public bool CheckBoxBool
        {
            get
            {
                return _checkBoxBool;
            }
            set
            {
                if (_checkBoxBool == value)
                    return;
                _checkBoxBool = value;
                NotifyOfPropertyChange();
            }
        }

        public BindingList<string> StringList { get; set; }

        public BindingList<Customer> Customers { get; set; }

        private void Form1_Load(object sender, System.EventArgs e)
        {
            PrepareTextBoxBinding();
            PrepareCheckBoxBinding();
            PrepareListBoxBinding();
            PrepareObjectListBoxBinding();
            PrepareDataGridBinding();
        }

        private void PrepareTextBoxBinding()
        {
            tbTextBoxBinding.DataBindings.Add("Text", this, "TextBoxText", false, DataSourceUpdateMode.OnPropertyChanged);
            lblTextBoxBinding.DataBindings.Add("Text", this, "TextBoxText", false, DataSourceUpdateMode.OnPropertyChanged);
        }

        private void PrepareCheckBoxBinding()
        {
            chbCheckBoxBinding.DataBindings.Add("Checked", this, "CheckBoxBool", false, DataSourceUpdateMode.OnPropertyChanged);
            lblCheckBoxBinding.DataBindings.Add("Text", this, "CheckBoxBool", false, DataSourceUpdateMode.OnPropertyChanged);
        }

        private void PrepareListBoxBinding()
        {
            StringList = new BindingList<string>();
            for (var i = 0; i <= 3; i++)
                StringList.Add($"Item {i}");
            lbSimpleStringBinding.DataSource = StringList;
        }

        private void PrepareObjectListBoxBinding()
        {
            Customers = new BindingList<Customer>();
            Customers.Add(new Customer("John", "Doe"));
            Customers.Add(new Customer("Karen", "Hill"));
            Customers.Add(new Customer("Max", "Mustermann"));
            // or set a different DisplayMember
            // lbListBoxObjectBinding.DisplayMember = "FirstName";
            lbListBoxObjectBinding.DataSource = Customers;
        }

        private void PrepareDataGridBinding()
        {
            dgvCustomers.DataSource = Customers;
        }

        private void NotifyOfPropertyChange([CallerMemberName] string propertyName = null)
        {
            if (propertyName == null)
                return;
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
